@REM
@REM ID: $Id: wlmjar.cmd,v 1.4 1999/06/09 21:34:00 amber Exp $
@REM
@REM COMPONENT_NAME: JMON WLM
@REM
@REM ORIGINS: Transarc Corp.
@REM
@REM (C) COPYRIGHT Transarc Corp. 1998
@REM All Rights Reserved
@REM Licensed Materials - Property of Transarc
@REM
@REM US Government Users Restricted Rights - Use, duplication or
@REM disclosure restricted by GSA ADP Schedule Contract with Transarc Corp
@REM
@REM
@REM HISTORY
@REM $TALog: wlmjar.cmd,v $

@REM Indrajit Poddar 8/16/99
@REM Changes made for robustness:
@REM 1. Added exit status checks after every command execution of critical 
@REM    comands
@REM 2. Added cleanup code for removal of resources created during execution
@REM 3. User Inputs for interface names are validated by checking the 
@REM    existence of of the interface class file in the specified jar file   
@REM 4. for including iioptools.jar, iioptr.jar, ujc.jar, the strategy used:
@REM     read setupCmdLine.bat assumming that WAS_HOME/bin exists in the PATH
@REM     if WAS_HOME & JAVA_HOME is set, add the jars to the classpath relative 
@REM        to WAS_HOME and classes.zip relative to JAVA_HOME
@REM     else flash a warning and use rest of CLASSPATH
@REM 
@REM 5. At the end, a sanity check is made on the newly created jar file to
@REM    check if the stub & baseStub.[java|class] files exist and are of 
@REM    non-zero sizes

::
:: CMVC Defect 88378
:: Thien Pham 10/31/00
:: remove all references iioptools.jar which was obsolete in release 3.5 
::
:: CMVC Defect 58199
:: Graeme Dixon 7/27/99
:: Correct jar files and cleanup on exit
::
:: CMVC Defect 55202
:: Amber Roy-Chowdhury IBM Transarc 6/16/99
:: Use pwd instead of cd in MKS mode
:: Use - instead of / as command line option flag
:: Add the jarfile to the classpath
:: 
:: Revision 1.4  1999/06/09  21:34:00  amber
:: [ Bug fixes ]
::
:: [ Do not use mksnt commands. Fix usage output. Other minor bug fixes ]
:: [from r1.3 by delta amber-24516-jmon-wlm-add-tool-for-wlming-client-side-jar-files, r1.4]
::
:: Revision 1.3  1999/06/02  21:40:28  amber
:: [ Fix bug ]
::
:: [ Was not recomipling the newly generated Stub and BaseStub ]
:: [from r1.2 by delta amber-24516-jmon-wlm-add-tool-for-wlming-client-side-jar-files, r1.3]
::
:: Revision 1.2  1999/06/02  19:12:04  amber
:: [ Added \M option to reuse existing manifest file ]
:: [from r1.1 by delta amber-24516-jmon-wlm-add-tool-for-wlming-client-side-jar-files, r1.2]
::
:: Revision 1.1  1999/05/24  19:14:03  amber
:: [ Added a tool to process client side jar files by running rmic and wlmsg to generate
::   wlm-aware stubs ]
::
:: [ WinNT version of tool ]
:: [added by delta amber-24516-jmon-wlm-add-tool-for-wlming-client-side-jar-files, r1.1]
::
@REM


@rem Takes a client jar file, extracts the remote interfaces out of it, runs rmic and wlmsg
@rem on the remore interfaces, and repackages the jar file.

@echo off

rem do not affect the environment outside the script
setlocal

rem command line options 
set PACKAGE_OPT=-P
set INTERFACE_OPT=-IF
set VERBOSE_OPT=-V
set JARFILE_OPT=-J
set MANIFEST_OPT=-M
set MKS_OPT=-MKS

rem the following are for internal use only
set DEBUG_OPT=-DD
set ECHO_OPT=-EE
set KEEP_OPT=-KK
set VVERBOSE_OPT=-VV

set debug=
set /a ifCount=0
set interfaces=
set verbose=
set vverbose=
set jarfile=
set keep=
set terminate=
set reuseManifest=
rem DOS style pwd
set pwd=cd 
set savedclasspath=%CLASSPATH%

if {%1}=={} (
    echo ERROR: Invalid command line.
    call :printUsage
    goto :EOF
)

rem process arguments
:nextArg
    if {%1}=={} goto :doneArg
    call :processArg %1 %2
    if not {%terminate%}=={} goto :EOF
    shift /1
    if not {%extraShift%}=={} shift /1
goto :nextArg
:doneArg

rem check that jar file, package, and at least one remote interface were specified
if {%jarfile%}=={} (
    echo ERROR: jar file was not specified
    call :printUsage
    goto :EOF
)
if {%package%}=={} (
    echo ERROR: Package was not specified
    call :printUsage
    goto :EOF
)
if "%interfaces%"=="" (
    echo ERROR: at least one remote interface must be specified
    call :printUsage
    goto :EOF
)
if not exist %jarfile% (
    echo ERROR: jar file %jarfile% not found
    goto :EOF
)

rem Assuming that WAS_HOME/bin is in the system path
if exist setupCmdLine.bat (
    call setupCmdLine.bat
)

if {%WAS_HOME%} == {} goto :nohome
if {%JAVA_HOME%} == {} goto :nohome
    set PATH=%JAVA_HOME%\bin;%PATH%
    goto :pathdone

:nohome
    echo Warning: WAS_HOME and/or JAVA_HOME not set. 
    echo using environment CLASSPATH for locating jar files:
    echo iioprt.jar, ujc.jar, classes.zip
    echo ------------------------------------------------
    echo Environment CLASSPATH is %CLASSPATH%
    echo ------------------------------------------------
    echo Using environment PATH to locate iiop rmic and ioser.dll
    echo ------------------------------------------------
    echo Environment PATH is %PATH%
    echo ------------------------------------------------

:pathdone

if {%verbose%}=={} goto :startProcessing 
echo jarfile=%jarfile%
if not {%package%}=={} echo package=%package%
echo remote interfaces=%interfaces%
echo CLASSPATH=%CLASSPATH%
echo PATH=%PATH%
rem check if the interfaces and package exist in the jar file
if not {%verbose%}=={} echo Looking for classes %interfaces% in %jarfile%...
set tmp=%package:.=/%
for /f "tokens=1,8" %%i in ('"jar tvf %jarfile% 2>&1"') do (
 for  %%p in (%interfaces%) do (
   if {%%j} == {%tmp%/%%p.class} if not {%%i} == {0} set ex_%%p_=1 )
)

for  %%p in (%interfaces%) do if not defined ex_%%p_ (
echo Error: 1 or more of the specified interfaces and package:
echo Package: %package% and Interfaces: %interfaces%
echo do not exist in the jarfile %jarfile%
goto :cleanupandexit
)
if not {%verbose%}=={} echo found all classes %interfaces% in %jarfile%...

:startProcessing


rem create a working directory
for /f "delims=:, tokens=2" %%i in ('%pwd%') do set topDir=%%i
set topDir=%topDir:/=\%
set workingDir=_wlmjar
if not exist %workingDir% goto :createWorkingDir
echo Directory %workingDir% exists already: it will be removed and recreated. 
echo Hit CTRL-C now to terminate or 
pause
rmdir /s /q %workingDir%

:createWorkingDir
if not {%verbose%}=={} echo creating working directory ...
mkdir %workingDir%

if errorlevel 1 (
echo  Error creating working directory: %workingDir%
goto :cleanupandexit
)

rem extract jar file into directory
set jaroptions=xf
if not {%vverbose%}=={} set jaroptions=v%jaroptions%
copy %jarfile% %workingDir%
if errorlevel 1 (
echo  Error in copying jarfile:%jarfile% to working directory: %workingDir%
goto :cleanupandexit
)
cd %topDir%\%workingDir%
if errorlevel 1 (
echo  Error in changing to working directory: %workingDir%
goto :cleanupandexit
)
if not {%verbose%}=={} (
    echo extracting jarfile ...
    echo jar %jaroptions% %jarfile%
)
jar %jaroptions% %jarfile%
if errorlevel 1 (
echo  Error in extracting from jarfile:%jarfile% to working directory: %workingDir%
goto :cleanupandexit
)
del %jarfile%

rem Add the jar files we need to the classpath

if {%WAS_HOME%} == {} goto :nowashome
if {%JAVA_HOME%} == {} goto :nowashome
    set classpath=%WAS_HOME%\lib\ujc.jar;%WAS_HOME%\lib\iioprt.jar;%CLASSPATH%;.;%topDir%\%jarfile%;%topDir%\%workingDir%;%JAVA_HOME%\lib\classes.zip
    set PATH=%WAS_HOME%\bin;%PATH%
    goto :classpathdone

:nowashome
    set classpath=%CLASSPATH%;.;%topDir%\%jarfile%;%topDir%\%workingDir%;%JAVA_HOME%\lib\classes.zip

:classpathdone

rem process the interfaces one by one
if not {%verbose%}=={} echo processing interfaces ...
for %%i in (%interfaces%) do ( call :processInterface %%i %package% ) & (if not {%terminate%} == {} goto :cleanupandexit )


if not {%terminate%} == {} goto :cleanupandexit
    
rem create new jarfile
cd %topDir%\%workingDir%
if not {%reuseManifest%}=={} (
  copy META-INF\MANIFEST.MF %topDir%\_wlm_manifest
  if errorlevel 1 (
    echo  Error in copying manifest file META-INF\MANIFEST.MF to %topDir%\_wlm_manifest
    goto :cleanupandexit
    )
)
rmdir /s /q META-INF
set newjarfile=_wlm_%jarfile%
set jaroptions=cf
if not {%vverbose%}=={} set jaroptions=v%jaroptions%
if not {%reuseManifest%}=={} set jaroptions=m%jaroptions% %topDir%\_wlm_manifest
if not {%verbose%}=={} (
    echo constructing jarfile ...
    echo jar %jaroptions% ..\%newjarfile%
)
jar %jaroptions% ..\%newjarfile% *
if errorlevel 1 (
 echo  Error in creating new jarfile: ..\%newjarfile%
 goto :cleanupandexit
)

cd %topDir%
rem Check to see if the the new jar file created has the 
rem wlmized stub classes and sources and their file sizes are not zero
if not {%verbose%} == {} (
echo "checking the new jarfile: %newjarfile% for the following." 
echo "1. <interface>_Stub.[java|class] exists"
echo "2. <interface>_BaseStub.[java|class] exists"
echo "3. The file sizes are not zero"
)
set tmp=%package:.=/%

for /f "tokens=1,8" %%i in ('"jar tvf %newjarfile% 2>&1"') do (
for  %%p in (%interfaces%) do (
 (if {%%j} == {%tmp%/_%%p_Stub.class} if not {%%i} == {0} set ex_a_%%p_=1)  
 (if {%%j} == {%tmp%/_%%p_Stub.java} if not {%%i} == {0} set ex_b_%%p_=1 ) 
 (if {%%j} == {%tmp%/_%%p_BaseStub.class} if not {%%i} == {0} set ex_c_%%p_=1)
 (if {%%j} == {%tmp%/_%%p_BaseStub.java} if not {%%i} == {0} set ex_d_%%p_=1)
)
)

for  %%p in (%interfaces%) do if not defined ex_a_%%p_ (
echo %tmp%/_%%p_Stub.class not created properly
set terminate=1
) else if not defined ex_b_%%p_ ( 
echo %tmp%/_%%p_Stub.java not created properly
set terminate=1
) else if not defined ex_c_%%p_ ( 
echo %tmp%/_%%p_BaseStub.class not created properly
set terminate=1
) else if not defined ex_d_%%p_ (
echo %tmp%/_%%p_BaseStub.java not created properly
set terminate=1
)

if not {%terminate%} == {} (
echo ERROR: in creation of new Jar File:%newjarfile%
goto :cleanupandexit
)
if not {%verbose%} == {} echo check succesful...

rem remove working directory
if not {%keep%}=={} goto :EOF
if not {%verbose%}=={} echo removing working directory ...
rmdir /s /q %workingDir%
if not {%reuseManifest%}=={} del /q _wlm_manifest

endlocal
goto :EOF

:processArg
    if not {%debug%}=={} echo DEBUG entering :processArg
    set extraShift=
    if /i not {%1} == {%PACKAGE_OPT%} goto :opt2
    set package=%2
    set extraShift=1
    goto :EOF

:opt2
    if /i not {%1} == {%INTERFACE_OPT%} goto :opt3
    set interfaces=%interfaces% %2
    set /a ifCount=ifCount+1
    set extraShift=1
    goto :EOF

:opt3
    if /i not {%1} == {%JARFILE_OPT%} goto :opt4
    set jarfile=%2
    set extraShift=1
    goto :EOF

:opt4
    if /i not {%1} == {%VERBOSE_OPT%} goto :opt5
    set verbose=1
    goto :EOF

:opt5
    if /i not {%1} == {%VVERBOSE_OPT%} goto :opt6
    set verbose=1
    set vverbose=1
    goto :EOF

:opt6
    if /i not {%1} == {%DEBUG_OPT%} goto :opt7
    set verbose=1
    set debug=1
    goto :EOF

:opt7
    if /i not {%1} == {%ECHO_OPT%} goto :opt8
    echo on
    goto :EOF

:opt8
    if /i not {%1} == {%KEEP_OPT%} goto :opt9
    set keep=1
    goto :EOF

:opt9
    if /i not {%1} == {%MANIFEST_OPT%} goto :opt10
    set reuseManifest=1
    goto :EOF

:opt10
    if /i not {%1} == {%MKS_OPT%} goto :wrong_opt
    rem MKS style pwd
    set pwd=pwd
    goto :EOF

:wrong_opt
    echo ERROR: Unrecognized option: %1
    call :printUsage
    set terminate=1
goto :EOF

:printUsage
    if not {%debug%}=={} echo DEBUG entering :printUsage
    echo Usage: wlmjar ^<options^> 
    echo Options:
    echo     -J  ^<jar file^> (required)
    echo     -IF ^<remote interface ^> ( ^<bean^> or ^<bean^>Home, at least one required)
    echo     -P  ^<package^> (required)
    echo     -M  (optional: reuse manifest)
    echo     -V  verbose mode
goto :EOF

:processInterface
    if not {%debug%}=={} echo DEBUG entering :processInterface
    if not {%debug%}=={} echo DEBUG params %1, %2
    set tmp=%2
    set tmp=%tmp:.=\%
    set ifDir=%topDir%\%workingDir%\%tmp%
    if not {%debug%}=={} echo DEBUG ifDir=%ifDir%
    cd %ifDir%
    if errorlevel 1 (
      echo  Error in changing to directory %ifDir%
      set terminate=1
      goto :EOF
     )
    if not {%debug%}=={} %pwd%
    if exist %ifDir%\%1.class goto :rmic
    if exist %1.java goto :genClass
    echo ERROR: could not find %1.java or %1.class in jar file.
    set terminate=1
    goto :EOF
:genClass
    set javacOpts=-classpath "%classpath%"
    if not {%verbose%}=={} echo javac %javacOpts% %1.java
    javac %javacOpts% %1.java
    if errorlevel 1 (
      echo ERROR: cannot compile %1.java
      set terminate=1
      goto :EOF
     )
:rmic
    rem work around rmic bug
    rem if the stubs exist already, rmic will not attempt to recreate them
    rem this causes problems for us as we need the stub sources 
    rem so we need to force rmic to run with -keep
    rem so we remove the stubs if they are present
    set stubClass=_%1_Stub.class
    if not exist %stubClass% goto :runRmic
    if not {%verbose%}=={} echo removing old stub %stubClass% ...
    del %stubClass%
    if errorlevel 1 (
      echo ERROR: failed to remove stub leftover Stubclass %stubClass%
      echo ERROR: if the stubs exist already, rmic will not attempt to recreate them
      set terminate=1
      goto :EOF
     )
:runRmic
    set rmicOpts=-classpath "%classpath%" -keep -iiop -g -d %topDir%\%workingDir%
    if not {%vverbose%}=={} set rmicOpts=%rmicOpts% -verbose
    if not {%verbose%}=={} echo rmic %rmicOpts% %2.%1
    rmic %rmicOpts% %2.%1
    if errorlevel 1 (
      echo ERROR: failed to run rmic on %2.%1
      echo ERROR: Please check that you have the iiop version of rmic in your PATH
      set terminate=1
      goto :EOF
     )
    set localClasspath=%classpath%;%topDir%\%workingDir%
    set javaOpts=-classpath "%localClasspath%"
    set wlmsgOpts=-p %2 -name %1
    if not {%verbose%}=={} echo java %javaOpts% com.ibm.ejs.wlm.wlmsg.wlmsg %wlmsgOpts%
    java %javaOpts% com.ibm.ejs.wlm.wlmsg.wlmsg %wlmsgOpts%
    if errorlevel 1 (
      echo ERROR: failed to run com.ibm.ejs.wlm.wlmsg.wlmsg 
      echo check CLASSPATH
      set terminate=1
      goto :EOF
     )
    set javacOpts=-classpath "%localClasspath%" -d %topDir%\%workingDir% 
    if not {%verbose%}=={} echo javac %javacOpts% _%1_BaseStub.java
    javac %javacOpts% _%1_BaseStub.java 
    if errorlevel 1 (
      echo ERROR: failed to compile _%1_BaseStub.java 
      echo check CLASSPATH
      set terminate=1
      goto :EOF
     )
    if not {%verbose%}=={} echo javac %javacOpts% _%1_Stub.java
    javac %javacOpts% _%1_Stub.java 
    if errorlevel 1 (
      echo ERROR: failed to compile _%1_Stub.java 
      echo check CLASSPATH
      set terminate=1
      goto :EOF
     )
    if not {%debug%}=={} %pwd%
    if not {%debug%}=={} echo DEBUG exiting :processInterface
goto :EOF

:cleanupandexit
echo Error: in wlmjar  file creation...
echo Cleaning up...

if {%jarfile%} == {} set jarfile="Unknown"
if {%topDir%} == {}  for /f "delims=:, tokens=2" %%i in ('%pwd%') do set topDir=%%i
if {%workingDir%} == {} set workingDir=_wlmjar

if exist %topDir%\_wlm_%jarfile% del %topDir%\_wlm_%jarfile%

cd %topDir%

if exist  %topDir%\%workingDir% rmdir /s /q %workingDir%

endlocal
goto :EOF

