// 5648-C84, 5648-C83, (C) Copyright IBM Corporation, 1997, 2000
// All rights reserved. Licensed Materials Property of IBM
// Note to US Government users: Documentation related to restricted rights
// Use, duplication or disclosure is subject to restrictions set forth in GSA ADP Schedule with IBM Corp.
// This page may contain other proprietary notices and copyright information, the terms of which must be observed and followed.
//
// This program may be used, executed, copied, modified and distributed
// without royalty for the purpose of developing, using,
// marketing, or distributing.

/*****************************************************************************************************
 * ApplyXSLListener provides a buffered listener essential for capturing, and then subsequently
 * reporting, XML and XSL processor messages which may be of use in debugging XML+XSL processed at
 * the server.
 *
 *****************************************************************************************************/

package WebSphereSamples.Xml;

import java.io.*;
import org.xml.sax.*;
import org.apache.xalan.xpath.xml.ProblemListenerDefault.*;

public class ApplyXSLListener extends org.apache.xalan.xpath.xml.ProblemListenerDefault implements ErrorHandler
{

   /**
     * Output stream
     */
   private ByteArrayOutputStream outStream = new ByteArrayOutputStream();

   /**
     * Buffered output stream
     */
   public PrintWriter out = null;

   /**
     * Constructor.
     */
   public ApplyXSLListener()
   {
      out = new PrintWriter(new BufferedOutputStream(outStream), true);
   }

   /**
     * Receive notification of a warning.
     *
     * @param spe The warning information encapsulated in a SAX parse exception.
     */
   public void warning(SAXParseException spe)
   {
      out.println("Parser Warning: " + spe.getMessage());
   }

   /**
     * Receive notification of a recoverable error.
     *
     * @param spe The error information encapsulated in a SAX parse exception.
     */
   public void error(SAXParseException spe)
   {
      out.println("Parser Error: " + spe.getMessage());
   }

   /**
     * Receive notification of a non-recoverable error.
     *
     * @param spe The error information encapsulated in a SAX parse exception.
     * @exception SAXException Always thrown
     */
   public void fatalError(SAXParseException spe)
   throws SAXException
   {
      out.println("Parser Fatal Error: " + spe.getMessage());
      throw spe;
   }

   /**
     * Returns the buffered processing message(s).
     * @return Buffered processing message(s)
     */
   public String getMessage()
   {
      return outStream.toString();
   }
}

