<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" xml:space="default">
  <xsl:output method="xml" 
              omit-xml-declaration="no" 
              indent="yes" 
              doctype-system="$server_root$$dsep$bin$dsep$xmlconfig.dtd"/>
  <xsl:strip-space elements="*"/>
  <xsl:template match="/">
    <xsl:apply-templates select="web-app"/>
  </xsl:template>

  <xsl:template match="web-app">
    <websphere-sa-config>
      <node name="$nodeName$" action="locate">
        <application-server name="$serverName$" action="locate">
          <servlet-engine name="$servletEngineName$" action="locate">
            <web-application name="$webappName$" action="create">
              <xsl:apply-templates select="web-app/description"/>
              <document-root>$documentRoot$</document-root>
              <classpath>
                <path value="$classpath$"/>
              </classpath>
              <auto-reload>true</auto-reload>
              <reload-interval>9000</reload-interval>
              <root-uri>$rootUri$</root-uri>
              <xsl:apply-templates select="servlet"/>
              <xsl:if test="count(welcome-file-list) != 0">
                <xsl:apply-templates select="welcome-file-list"/>
              </xsl:if>  
              <xsl:if test="count(mime-mapping) != 0">
                <xsl:apply-templates select="mime-mapping"/>
              </xsl:if>  
              <xsl:if test="count(error-page) != 0">
                <xsl:apply-templates select="error-page"/>
              </xsl:if>  
              <xsl:if test="count(session-config) != 0">
                <xsl:apply-templates select="session-config"/>
              </xsl:if>  
              <xsl:if test="count(taglib) != 0">
                <xsl:apply-templates select="taglib"/>
              </xsl:if>  
              <xsl:if test="count(context-param) != 0">
                <group-attributes>
                  <xsl:apply-templates select="context-param"/>
                </group-attributes>
              </xsl:if>  
	      <servlet name="ErrorReporter" action="create">
                <description>Default error reporter servlet</description>
                <code>com.ibm.servlet.engine.webapp.DefaultErrorReporter</code>
                <load-at-startup>true</load-at-startup>
                <uri-paths>
                  <uri value="/ErrorReporter"/>
                </uri-paths>
	      </servlet>
              <servlet name="jsp11" action="create">
                <description>JSP 1.1 support servlet</description>
                <code>org.apache.jasper.runtime.JspServlet</code>
                <load-at-startup>true</load-at-startup>
                <uri-paths>
                  <uri value="*.jsp"/>
                  <uri value="*.jsv"/>
                  <uri value="*.jsw"/>
                </uri-paths>
              </servlet>
              <servlet name="file" action="create">
                <description>Enables File Serving from this Web Application</description>
                <code>com.ibm.servlet.engine.webapp.SimpleFileServlet</code>
                <init-parameters>
                  <parameter name="" value=""/>
                </init-parameters>
                <load-at-startup>true</load-at-startup>
                <uri-paths>
                  <uri value="/"/>
                </uri-paths>
              </servlet>
            </web-application>  
          </servlet-engine>
        </application-server>
      </node>
    </websphere-sa-config>
  </xsl:template>

  <xsl:template match="web-app/description">
    <description><xsl:value-of select="."/></description>
  </xsl:template>

  <xsl:template match="welcome-file-list">
    <welcome-file-list>
      <welcome-file><xsl:value-of select="welcome-file"/></welcome-file>
    </welcome-file-list>
  </xsl:template>

  <xsl:template match="mime-mapping">
    <mime-mapping>
      <extension><xsl:value-of select="extension"/></extension>
      <mime-type><xsl:value-of select="mime-type"/></mime-type>
    </mime-mapping>
  </xsl:template>

  <xsl:template match="session-config">
    <session-config>
      <session-timeout><xsl:value-of select="session-timeout"/></session-timeout>
    </session-config>
  </xsl:template>

  <xsl:template match="taglib">
    <taglib>
      <taglib-uri><xsl:value-of select="taglib-uri"/></taglib-uri>
      <taglib-location><xsl:value-of select="taglib-location"/></taglib-location>
    </taglib>
  </xsl:template>

  <xsl:template match="context-param">
    <attribute name="{param-name}" value="{param-value}"/>
  </xsl:template>

  <xsl:template match="error-page">
    <error-page-j2ee>
    <xsl:apply-templates select="error-code"/>
    <xsl:apply-templates select="exception-type"/>
    <location><xsl:value-of select="location"/></location>
    </error-page-j2ee>
  </xsl:template>

  <xsl:template match="error-code">
    <error-code><xsl:value-of select="."/></error-code>
  </xsl:template>

  <xsl:template match="exception-type">
    <exception-type><xsl:value-of select="."/></exception-type>
  </xsl:template>

 <xsl:template match="servlet">
    <servlet name="{servlet-name}" action="create">
      <xsl:apply-templates select="servlet/description"/>
      <xsl:apply-templates select="servlet-class"/>
      <init-parameters>
        <xsl:apply-templates select="init-param"/>
      </init-parameters>  
      <xsl:apply-templates select="load-on-startup"/>
      <uri-paths>
        <xsl:apply-templates select="//servlet-mapping">
          <xsl:with-param name="name-of-servlet">
            <xsl:value-of select="servlet-name"/>
          </xsl:with-param>
        </xsl:apply-templates>  
      </uri-paths>
    </servlet>
  </xsl:template>

  <xsl:template match="servlet/description">
    <description><xsl:value-of select="."/></description>
  </xsl:template>

  <xsl:template match="servlet-class">
    <code><xsl:value-of select="."/></code>
  </xsl:template>

  <xsl:template match="load-on-startup">
    <load-at-startup>true</load-at-startup>
  </xsl:template>

  <xsl:template match="init-param">
    <parameter name="{param-name}" value="{param-value}"/>
  </xsl:template>

  <xsl:template match="//servlet-mapping">
    <xsl:param name="name-of-servlet"/>
    <xsl:variable name="sn" select="servlet-name"/>
    <xsl:if test="$name-of-servlet=$sn">
      <uri value="{url-pattern}"/>
    </xsl:if>  
  </xsl:template>

</xsl:stylesheet>
